import json
import logging

import docker

from concierge.config import settings
from concierge.executor.base import BaseExecutor
from concierge.task import Task

logger = logging.getLogger(__name__)


class DockerExecutor(BaseExecutor):
    def __init__(self, identity: str):
        self.client = docker.from_env()
        self.identity = identity

    def labels_for_task(self, uuid: str):
        return {
            "concierge.chaoswest.tv/concierge-id": self.identity,
            "concierge.chaoswest.tv/task-id": uuid,
        }

    def run_task(self, task: Task):
        logger.debug(task)
        logger.info("Running task %s of type %s", task.uuid, task.type)
        labels = self.labels_for_task(task.uuid)
        container = self.client.containers.run(
            settings.task_types[task.type]["image"],
            cpu_shares=int(
                float(
                    settings.task_types[task.type]
                    .get("reservations", {})
                    .get("cpus", 0.25)
                )
                * 1024
            ),
            mem_limit=settings.task_types[task.type]
            .get("reservations", {})
            .get("memory", "256m"),
            detach=True,
            labels=labels,
            restart_policy={"Name": "always"},
            name=f"concierge-task-{task.uuid}",
            environment={"CONCIERGE_CONFIGURATION": json.dumps(task.configuration)},
        )
        logger.debug(container)
        logger.info("Started container %s for task %s", container.name, task.uuid)

    def list_task_uuids(self) -> list[str]:
        labelfilters = [f"concierge.chaoswest.tv/concierge-id={self.identity}"]
        containers = self.client.containers.list(filters={"label": labelfilters})
        return [c.labels["concierge.chaoswest.tv/task-id"] for c in containers]

    def stop_task(self, uuid: str):
        logger.info("Stopping task %s", uuid)
        labels = self.labels_for_task(uuid)
        labelfilters = [f"{k}={v}" for k, v in labels.items()]

        containers = self.client.containers.list(filters={"label": labelfilters})

        for container in containers:
            logger.debug(container)
            logger.info("Stopping container %s for task %s", container.name, uuid)
            container.stop()
            container.remove()
