"""Main module for the concierge service, contains the main loop and signal handlers."""

import logging
import signal
import sys
import time

from requests.exceptions import RequestException

from concierge.config import settings
from concierge.executor.docker import DockerExecutor
from concierge.portier import PortierClient

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

executor = DockerExecutor(settings.identity)
portier = PortierClient(settings.identity, settings.portier_endpoint)

grace = settings.loop_grace


def cleanup_tasks():
    """Stop and release all running tasks."""

    running_tasks = executor.list_task_uuids()
    for task_uuid in running_tasks:
        executor.stop_task(task_uuid)
        try:
            portier.release_task(task_uuid)
        except RequestException as e:
            logger.error(
                "Failed to release task %s while shutting down: %s", task_uuid, e
            )
    logger.info("All tasks stopped and released")


def sigterm_handler(signum, _):
    """Signal handler for SIGTERM and SIGINT, stops all running tasks and exits."""

    logger.info("Received %s, shutting down", signum)
    cleanup_tasks()
    sys.exit(0)


signal.signal(signal.SIGTERM, sigterm_handler)
signal.signal(signal.SIGINT, sigterm_handler)


while True:
    time.sleep(settings.loop_interval)

    try:
        tasks_offered, tasks_current = portier.heartbeat()
        grace = settings.loop_grace
    except RequestException:
        if grace <= 0:
            logger.error("portier remains unreachable, exiting")
            cleanup_tasks()
            sys.exit(1)
        else:
            logger.warning("portier unreachable, will try %d more times", grace)
            grace -= 1
            continue

    for offer in tasks_offered:

        logger.info("Offered task %s of type %s", offer.uuid, offer.type)
        if offer.type not in settings.task_types.keys():
            logger.info("Task type %s not supported, skipping", offer.type)
            continue

        try:
            task = portier.claim_task(offer)
        except RequestException:
            continue

        if task is None:
            logger.info("Unable to claim task, maybe someone else was quicker?")
            continue

        tasks_current.append(task.uuid)

        try:
            executor.run_task(task)
        except Exception as e:  # pylint: disable=broad-except
            logger.error("Failed to start task: %s", e)

            try:
                portier.release_task(task.uuid)
            except RequestException:
                pass

            tasks_current.remove(task.uuid)

    for task_uuid in executor.list_task_uuids():
        if task_uuid not in tasks_current:
            try:
                executor.stop_task(task_uuid)
            except Exception as e:  # pylint: disable=broad-except
                logger.error("Failed to stop obsolete task: %s", e)
