"""Client class that communicates with Portier"""

import logging

import requests

from concierge import task

logger = logging.getLogger(__name__)


class PortierClient:
    def __init__(self, concierge_id: str, base_url: str):
        self.concierge_id = concierge_id
        self.base_url = base_url

    def _request(self, method: str, endpoint: str, **kwargs) -> dict:
        r = requests.request(
            method,
            f"{self.base_url}/concierge/api/{self.concierge_id}/{endpoint}",
            **kwargs,
        )
        r.raise_for_status()
        return r.json()

    def heartbeat(self) -> tuple[list[task.TaskOffer], list[str]]:
        logger.debug("Heartbeat")
        response = self._request("POST", "heartbeat")

        if response.get("success", False):
            return [task.TaskOffer(**t) for t in response.get("available", [])], [
                t.get("uuid") for t in response.get("claims", [])
            ]
        if response.get("error"):
            logger.error(response.get("error"))
        else:
            logger.error("Unknown error")
            logger.debug(response)
        return [], []

    def claim_task(self, offer: task.TaskOffer) -> task.Task | None:
        logger.debug("Claiming task %s", offer.uuid)
        response = self._request("POST", f"claim/{offer.uuid}")
        if response.get("success", False):
            logger.debug("Task %s claimed", response["uuid"])
            return task.Task(
                uuid=response["uuid"],
                type=response["type"],
                configuration=response["configuration"],
            )
        logger.debug("Failed to claim task %s", offer.uuid)
        if response.get("error"):
            logger.error(response.get("error"))
        else:
            logger.error("Unknown error")
            logger.debug(response)
        return None

    def release_task(self, uuid: str):
        logger.debug("Releasing task %s", uuid)
        return self._request("POST", f"release/{uuid}")
